/***********************************************************
Copyright 1996 Roberto Alsina

			All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the names of Stichting Mathematisch
Centrum or CWI not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior permission.

STICHTING MATHEMATISCH CENTRUM DISCLAIMS ALL WARRANTIES WITH REGARD TO
THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH CENTRUM BE LIABLE
FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

******************************************************************/

#include "Python.h"
#include "forms.h"
static PyObject *ErrorObject;

/* ----------------------------------------------------- */

static char bmp_create__doc__[] =
    "Creates a bitmap"
    ;

static PyObject *
bmp_create(self, args)
PyObject *self;			/* Not used */
PyObject *args;
{
    int t, x, y, w, h;
    char *l;
    FL_OBJECT *o;

    if (!PyArg_ParseTuple(args, "iiiiis", &t, &x, &y, &w, &h, &l))
        return NULL;
    o = fl_create_bitmap(t, x, y, w, h, l);
    return Py_BuildValue("l", (long) o);
}

static char bmp_set_data__doc__[] =
    "Sets the bitmap's data"
    ;

static PyObject *
bmp_set_data(self, args)
PyObject *self;			/* Not used */
PyObject *args;
{
    long o;
    int w, h;
    unsigned char *b;

    if (!PyArg_ParseTuple(args, "liis", &o, &w, &h, &b))
        return NULL;
    fl_set_bitmap_data((FL_OBJECT *) o, w, h, b);
    Py_INCREF(Py_None);
    return Py_None;
}



static char bmp_set_file__doc__[] =
    "Loads a file into a bitmap"
    ;

static PyObject *
bmp_set_file(self, args)
PyObject *self;			/* Not used */
PyObject *args;
{
    long o;
    char *f;

    if (!PyArg_ParseTuple(args, "ls", &o, &f))
        return NULL;
    fl_set_bitmap_file((FL_OBJECT *) o, f);
    Py_INCREF(Py_None);
    return Py_None;
}


/* List of methods defined in the module */

static struct PyMethodDef bmp_methods[] =
{
    {"create", bmp_create, 1, bmp_create__doc__},
    {"set_data", bmp_set_data, 1, bmp_set_data__doc__},
    {"set_file", bmp_set_file, 1, bmp_set_file__doc__},

    {NULL, NULL}		/* sentinel */
};


/* Initialization function for the module (*must* be called initbitmap) */

static char bitmaps_module_documentation[] =
    ""
    ;


static struct PyModuleDef moduledef =
{
    PyModuleDef_HEAD_INIT,
    "bitmaps",
    bitmaps_module_documentation,
    -1,
    bmp_methods,
    NULL,
    NULL,
    NULL,
    NULL,
};

PyMODINIT_FUNC
PyInit_bitmaps()
{
    PyObject *m, *d;

    /* Create the module and add the functions */
    m = PyModule_Create(&moduledef);

    /* Add some symbolic constants to the module */
    d = PyModule_GetDict(m);
    ErrorObject = PyBytes_FromString("bitmaps.error");
    PyDict_SetItemString(d, "error", ErrorObject);

    /* XXXX Add constants here */

    /* Check for errors */
    if (PyErr_Occurred())
        Py_FatalError("can't initialize module bitmaps");
}
